@@ -225,6 +225,36 @@ class HttpClientTest(testtools.TestCase):
         self.assertEqual({}, body)
         self.m.VerifyAll()
 
+    def test_http_json_request_argument_passed_to_requests(self):
+        """Check that we have sent the proper arguments to requests."""
+        # Record a 200
+        mock_conn = http.requests.request(
+            'GET', 'http://example.com:8004',
+            allow_redirects=False,
+            cert=('RANDOM_CERT_FILE', 'RANDOM_KEY_FILE'),
+            verify=True,
+            data='"text"',
+            headers={'Content-Type': 'application/json',
+                     'Accept': 'application/json',
+                     'X-Auth-Url': 'http://AUTH_URL',
+                     'User-Agent': 'python-heatclient'})
+        mock_conn.AndReturn(
+            fakes.FakeHTTPResponse(
+                200, 'OK',
+                {'content-type': 'application/json'},
+                '{}'))
+        # Replay, create client, assert
+        self.m.ReplayAll()
+        client = http.HTTPClient('http://example.com:8004')
+        client.verify_cert = True
+        client.cert_file = 'RANDOM_CERT_FILE'
+        client.key_file = 'RANDOM_KEY_FILE'
+        client.auth_url = 'http://AUTH_URL'
+        resp, body = client.json_request('GET', '', data='text')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual({}, body)
+        self.m.VerifyAll()
+
     def test_http_json_request_w_req_body(self):
         # Record a 200
         mock_conn = http.requests.request(
@@ -385,6 +415,38 @@ class HttpClientTest(testtools.TestCase):
         self.assertEqual(200, resp.status_code)
         self.m.VerifyAll()
 
+    def test_http_manual_redirect_put_uppercase(self):
+        mock_conn = http.requests.request(
+            'PUT', 'http://EXAMPLE.com:8004/foo',
+            allow_redirects=False,
+            headers={'Content-Type': 'application/json',
+                     'Accept': 'application/json',
+                     'User-Agent': 'python-heatclient'})
+        mock_conn.AndReturn(
+            fakes.FakeHTTPResponse(
+                302, 'Found',
+                {'location': 'http://example.com:8004/foo/bar'},
+                ''))
+        mock_conn = http.requests.request(
+            'PUT', 'http://EXAMPLE.com:8004/foo/bar',
+            allow_redirects=False,
+            headers={'Content-Type': 'application/json',
+                     'Accept': 'application/json',
+                     'User-Agent': 'python-heatclient'})
+        mock_conn.AndReturn(
+            fakes.FakeHTTPResponse(
+                200, 'OK',
+                {'content-type': 'application/json'},
+                '{}'))
+
+        self.m.ReplayAll()
+
+        client = http.HTTPClient('http://EXAMPLE.com:8004/foo')
+        resp, body = client.json_request('PUT', '')
+
+        self.assertEqual(200, resp.status_code)
+        self.m.VerifyAll()
+
     def test_http_manual_redirect_prohibited(self):
         mock_conn = http.requests.request(
             'DELETE', 'http://example.com:8004/foo',
@@ -403,6 +465,24 @@ class HttpClientTest(testtools.TestCase):
                           client.json_request, 'DELETE', '')
         self.m.VerifyAll()
 
+    def test_http_manual_redirect_error_without_location(self):
+        mock_conn = http.requests.request(
+            'DELETE', 'http://example.com:8004/foo',
+            allow_redirects=False,
+            headers={'Content-Type': 'application/json',
+                     'Accept': 'application/json',
+                     'User-Agent': 'python-heatclient'})
+        mock_conn.AndReturn(
+            fakes.FakeHTTPResponse(
+                302, 'Found',
+                {},
+                ''))
+        self.m.ReplayAll()
+        client = http.HTTPClient('http://example.com:8004/foo')
+        self.assertRaises(exc.InvalidEndpoint,
+                          client.json_request, 'DELETE', '')
+        self.m.VerifyAll()
+
     def test_http_json_request_redirect(self):
         # Record the 302
         mock_conn = http.requests.request(
@@ -432,8 +512,8 @@ class HttpClientTest(testtools.TestCase):
         self.m.ReplayAll()
         client = http.HTTPClient('http://example.com:8004')
         resp, body = client.json_request('GET', '')
-        self.assertEqual(resp.status_code, 200)
-        self.assertEqual(body, {})
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual({}, body)
         self.m.VerifyAll()
 
     def test_http_404_json_request(self):
@@ -503,14 +583,15 @@ class HttpClientTest(testtools.TestCase):
         mock_logging_debug = logging.Logger.debug(
             "curl -i -X GET -H 'key: value' --key TEST_KEY "
             "--cert TEST_CERT --cacert TEST_CA "
-            "-k http://foo/bar"
+            "-k -d 'text' http://foo/bar"
         )
         mock_logging_debug.AndReturn(None)
         self.m.ReplayAll()
 
         client = http.HTTPClient('http://foo')
         client.ssl_connection_params = ssl_connection_params
-        client.log_curl_request('GET', '/bar', {'headers': headers})
+        client.log_curl_request('GET', '/bar', {'headers': headers,
+                                                'data': 'text'})
 
         self.m.VerifyAll()
 
@@ -540,6 +621,27 @@ class HttpClientTest(testtools.TestCase):
                           client._http_request, "/", "GET")
         self.m.VerifyAll()
 
+    def test_http_request_specify_timeout(self):
+        mock_conn = http.requests.request(
+            'GET', 'http://example.com:8004',
+            allow_redirects=False,
+            headers={'Content-Type': 'application/json',
+                     'Accept': 'application/json',
+                     'User-Agent': 'python-heatclient'},
+            timeout=float(123))
+        mock_conn.AndReturn(
+            fakes.FakeHTTPResponse(
+                200, 'OK',
+                {'content-type': 'application/json'},
+                '{}'))
+        # Replay, create client, assert
+        self.m.ReplayAll()
+        client = http.HTTPClient('http://example.com:8004', timeout='123')
+        resp, body = client.json_request('GET', '')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual({}, body)
+        self.m.VerifyAll()
+
     def test_get_system_ca_file(self):
         chosen = '/etc/ssl/certs/ca-certificates.crt'
         self.m.StubOutWithMock(os.path, 'exists')
@@ -547,7 +649,7 @@ class HttpClientTest(testtools.TestCase):
         self.m.ReplayAll()
 
         ca = http.get_system_ca_file()
-        self.assertEqual(ca, chosen)
+        self.assertEqual(chosen, ca)
 
         self.m.VerifyAll()
 
@@ -557,17 +659,17 @@ class HttpClientTest(testtools.TestCase):
 
     def test_passed_cert_to_verify_cert(self):
         client = http.HTTPClient('https://foo', ca_file="NOWHERE")
-        self.assertEqual(client.verify_cert, "NOWHERE")
+        self.assertEqual("NOWHERE", client.verify_cert)
 
         self.m.StubOutWithMock(http, 'get_system_ca_file')
         http.get_system_ca_file().AndReturn("SOMEWHERE")
         self.m.ReplayAll()
         client = http.HTTPClient('https://foo')
-        self.assertEqual(client.verify_cert, "SOMEWHERE")
+        self.assertEqual("SOMEWHERE", client.verify_cert)
 
     def test_curl_log_i18n_headers(self):
         self.m.StubOutWithMock(logging.Logger, 'debug')
-        kwargs = {'headers': {'Key': 'foo\xe3\x8a\x8e'}}
+        kwargs = {'headers': {'Key': b'foo\xe3\x8a\x8e'}}
 
         mock_logging_debug = logging.Logger.debug(
             u"curl -i -X GET -H 'Key: foo㊎' http://somewhere"
