@@ -44,7 +44,8 @@ class StackStatusActionTest(testtools.TestCase):
         ('UPDATE', dict(action='UPDATE')),
         ('ROLLBACK', dict(action='ROLLBACK')),
         ('SUSPEND', dict(action='SUSPEND')),
-        ('RESUME', dict(action='RESUME'))
+        ('RESUME', dict(action='RESUME')),
+        ('CHECK', dict(action='CHECK'))
     ], [
         ('IN_PROGRESS', dict(status='IN_PROGRESS')),
         ('FAILED', dict(status='FAILED')),
@@ -98,6 +99,12 @@ class StackOperationsTest(testtools.TestCase):
         stack = stack.create()
         manager.create.assert_called_once_with('the_stack/abcd1234')
 
+    def test_preview_stack(self):
+        manager = MagicMock()
+        stack = mock_stack(manager, 'the_stack', 'abcd1234')
+        stack = stack.preview()
+        manager.preview.assert_called_once_with()
+
 
 class StackManagerNoPaginationTest(testtools.TestCase):
 
